@@ -30,7 +30,7 @@ network.layout.animate.kamadakawai <-function(net, dist.mat=NULL, default.dist=N
 # If your package is non-commercial, I see no problem in including MDSJ in your implementation.
 # Please feel free to contact me regarding technical MDS issues 
 
-network.layout.animate.MDSJ <-function(net, dist.mat=NULL, default.dist=NULL,seed.coords=NULL, layout.par=list(max_iter=50),verbose=TRUE){
+network.layout.animate.MDSJ <-function(net, dist.mat=NULL, default.dist=NULL,seed.coords=NULL, layout.par=list(max_iter=50,dimensions=2),verbose=TRUE){
   # check that mdsj library can be located
   mdsj.path <- check.mdsj()
   
@@ -42,6 +42,10 @@ network.layout.animate.MDSJ <-function(net, dist.mat=NULL, default.dist=NULL,see
   
   n <- network.size(net)
   max_iter <- 50 #layout.par$max_iter
+  dim<-2  # default to two dimeions
+  if (!is.null(layout.par$dimensions)){  # if dimensions specified in layout par, grab 'em
+    dim<-as.numeric(layout.par$dimensions)
+  }
   #if seed.coord already set in layoutpar, overide
   if (is.null(dist.mat)){
     dist.mat <- layout.distance(net,default.dist=default.dist)
@@ -53,7 +57,11 @@ network.layout.animate.MDSJ <-function(net, dist.mat=NULL, default.dist=NULL,see
   coord.file <- tempfile("coords",fileext=".txt")
   write.matrix(dist.mat,file=filename)
   if (!is.null(seed.coords)){
-    write.table(seed.coords,file=coord.file,col.names=FALSE,row.names=FALSE)
+    # limit seeds coords to the specified number of dimensions
+    if(ncol(seed.coords)!=dim){
+      warning('Dimensions of seed coord matrix does not match dimensions specified in layout.par ',dim,'. Only the first ',dim,' columns will be used.')
+    }
+    write.table(seed.coords[,seq.int(dim)],file=coord.file,col.names=FALSE,row.names=FALSE)
   } else {
     coord.file <- ""
   }
@@ -61,10 +69,15 @@ network.layout.animate.MDSJ <-function(net, dist.mat=NULL, default.dist=NULL,see
   if (.Platform$OS.type=='windows'){
     sep<-';'
   }
+  verbosity<-0  # make a verbose flag for java
+  if(verbose){
+    verbosity<-1
+  }
+ 
   #command = paste("java -jar",mdsjpath,"-e-2",filename)
   #TODO java classpath is platform dependent, need to modify?  
   #/home/skyebend/SNA_health:/home/skyebend/SNA_health/mdsj.jar MDSJWrapper  
-  command = paste("java -cp ",paste(mdsj.path,file.path(mdsj.path,"mdsj.jar"),sep=sep),"MDSJWrapper", n,max_iter,filename,coord.file)
+  command = paste("java -cp ",paste(mdsj.path,file.path(mdsj.path,"mdsj.jar"),sep=sep),"MDSJWrapper", n,dim,verbosity,max_iter,filename,coord.file)
   #print(command)
   output <- system(command,intern=TRUE)
   if(verbose){
@@ -77,9 +90,10 @@ network.layout.animate.MDSJ <-function(net, dist.mat=NULL, default.dist=NULL,see
   #NEED TO CHECK FOR ERROR
   #only grab the last n lines
   coords <- NULL
-  if(length(output)>n){
-    coords <- matrix(data=as.numeric(unlist(strsplit(output[(length(output)-nrow(dist.mat)+1):length(output)]," "))),ncol=2,byrow=TRUE)
+  if(length(output)>=n){
+    coords <- matrix(data=as.numeric(unlist(strsplit(output[(length(output)-nrow(dist.mat)+1):length(output)]," "))),ncol=dim,byrow=TRUE)
   } else {
+    # something must be wrong, so print the output (unless it would already have been printed)
     if (!verbose){print(output)}
     error=stop("Unable to parse coordinates returned MDSJ java code")
   }
@@ -109,23 +123,55 @@ network.layout.animate.Graphviz <-function(net, dist.mat=NULL, default.dist=NULL
    } else {
      gv.args<-layout.par$gv.args
    }
+   
+   # need to determine if we will be including weights from the distance matrix, the edge attributes
+   # or neither
+   gv.len.mode<-layout.par$gv.len.mode
+   if (is.null(gv.len.mode)){
+     gv.len.mode<-'gv.edge.len'
+   }else{
+     gv.len.mode<-match.arg(gv.len.mode,c('gv.edge.len','ndtv.distance.matrix'))
+   }
+   
+   # possible list of edge attributes from the network to include
+   gv.edge.attrs<-layout.par$gv.edge.attrs
     
   } else {
     # set to defaults
     gv.engine<-'neato'
+    gv.len.mode<-'gv.edge.len'
+    gv.edge.attrs=NULL
   }
   
+  
+  
+  
   n <- network.size(net)
   #if seed.coord already set in layoutpar, overide
-  if (!is.null(dist.mat)){
-    warning("distance matrix not currently implented for Graphviz layouts")
+  if (!is.null(dist.mat) & gv.len.mode=='gv.edge.len'){
+    warning("distance matrix was ignored because gv.len.mode=='gv.edge.len'")
   }
+  
   if (is.null(default.dist)){
     default.dist<-FALSE
   }
 
   filename <- tempfile("network",fileext=".dot")
-  export.dot(net,filename,coords=seed.coords,all.dyads=default.dist) #TODO: include attributes if defined
+  
+  if (gv.len.mode=='ndtv.distance.matrix'){
+    if (is.null(dist.mat)){
+      stop("dist.mat must not be null to use the gv.len.mode=='ndtv.distance.matrix' option")
+    }
+    # pass it the distance matrix as len, ignore edge attributes
+    export.dot(net,filename,coords=seed.coords,all.dyads=dist.mat, edge.attrs=gv.edge.attrs) 
+  } else if (!is.null(default.dist)) {
+    # pass in the default dist to create distance matrix as len, ignore edge attributes        
+    export.dot(net,filename,coords=seed.coords,all.dyads=default.dist, edge.attrs=gv.edge.attrs) 
+  } else {  #(gv.len.mode=='gv.edge.len')
+    #include edge attributes (especially 'len' ) if defined so that gv can uses them
+    export.dot(net,filename,coords=seed.coords,edge.attrs = gv.edge.attrs)
+  }
+  #TODO: should be able to pipe .dot contents directly into GV and skip disk IO ?
   command <- paste(gv.engine,"-Tplain",gv.args,filename)
   #print(command)
   output <- system(command,intern=TRUE)
